/*
 * Copyright 2005 - 2009  Zarafa B.V.
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License, version 3, 
 * as published by the Free Software Foundation with the following additional 
 * term according to sec. 7:
 *  
 * According to sec. 7 of the GNU Affero General Public License, version
 * 3, the terms of the AGPL are supplemented with the following terms:
 * 
 * "Zarafa" is a registered trademark of Zarafa B.V. The licensing of
 * the Program under the AGPL does not imply a trademark license.
 * Therefore any rights, title and interest in our trademarks remain
 * entirely with us.
 * 
 * However, if you propagate an unmodified version of the Program you are
 * allowed to use the term "Zarafa" to indicate that you distribute the
 * Program. Furthermore you may use our trademarks where it is necessary
 * to indicate the intended purpose of a product or service provided you
 * use it in accordance with honest practices in industrial or commercial
 * matters.  If you want to propagate modified versions of the Program
 * under the name "Zarafa" or "Zarafa Server", you may only do so if you
 * have a written permission by Zarafa B.V. (to acquire a permission
 * please contact Zarafa at trademark@zarafa.com).
 * 
 * The interactive user interface of the software displays an attribution
 * notice containing the term "Zarafa" and/or the logo of Zarafa.
 * Interactive user interfaces of unmodified and modified versions must
 * display Appropriate Legal Notices according to sec. 5 of the GNU
 * Affero General Public License, version 3, when you propagate
 * unmodified or modified versions of the Program. In accordance with
 * sec. 7 b) of the GNU Affero General Public License, version 3, these
 * Appropriate Legal Notices must retain the logo of Zarafa or display
 * the words "Initial Development by Zarafa" if the display of the logo
 * is not reasonably feasible for technical reasons. The use of the logo
 * of Zarafa in Legal Notices is allowed for unmodified and modified
 * versions of the software.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 *  
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

/**
 * Z-Merge Archive Plugin
 *
 * Author: Sebastian Kummer <s.kummer@zarafa.com>
 *
 */
Pluginzma.prototype = new Plugin;
Pluginzma.prototype.constructor = Pluginzma;
Pluginzma.superclass = Plugin.prototype;

function Pluginzma(){}

/**
 * init
 *
 * registers for events 
 */
Pluginzma.prototype.init = function(){
    this.registerHook("client.module.maillistmodule.contextmenu.buildup");
    this.registerHook("client.module.maillistmodule.execute.before");
}

/**
 * execute
 *
 * is called when registered events are triggered 
 */
Pluginzma.prototype.execute = function(eventID, data){
    switch(eventID){
        case "client.module.maillistmodule.contextmenu.buildup":
            this.addArchiveContextMenu(data);
            break;
        case "client.module.maillistmodule.execute.before":
            this.mailListModuleExecute(data);
            break;
    }
}

/**
 * Event: addArchiveContextMenu
 *
 * adds new entries to the contextmenu
 */
Pluginzma.prototype.addArchiveContextMenu = function(data){
    var items = data["contextmenu"];

    if (window.zmaprofiles) {
        items.push(webclient.menu.createMenuItem("seperator", ""));    

        for(var profilename in window.zmaprofiles) {
            items.push(webclient.menu.createMenuItem("zma_"+profilename, window.zmaprofiles[profilename]+"...", false, Pluginzma.prototype.eventListContextArchiveEmail, false, false, {profile: profilename}));
        }
    }
    else alert(dgettext("plugin_zma", "Z-Merge Archive: invalid configuration"));
}

/**
 * Event: eventListContextArchiveEmail
 *
 * directly called (from maillistmodule) when the inserted events are triggered
 * contacts server to start archivation
 */
Pluginzma.prototype.eventListContextArchiveEmail = function(moduleObject, element, event){
    // Hide menu
    element.parentNode.style.display = "none";
    
    var data = new Array()
    data["profile"] = element.data['profile'];
    data["store"] = moduleObject.storeid;
    data["parententryid"] = moduleObject.entryid;
    data["entryid"] = moduleObject.entryids[element.parentNode.elementid];
    webclient.xmlrequest.addData(this, "zmaSendMail", data);

    // deselect selected message
    eventListCheckSelectedContextMessage(this);
}

/**
 * Event: mailListModuleExecute
 *
 * process incoming execute events for maillistmodule
 */
Pluginzma.prototype.mailListModuleExecute = function(data) {
    var type = data["type"];
    var action = data["action"];

    switch(type){
        case "archiveResponse":
            this.archiveResponse(action);
            break;
    }
}

/**
 * Event: archiveResponse
 *
 * process responses from the 3rd party server
 */
Pluginzma.prototype.archiveResponse = function(action) {
    var status = dhtml.getXMLValue(action, "status", "none");
    var message = dhtml.getXMLValue(action, "message", "none");
    var serverurl = dhtml.getXMLValue(action, "serverurl", "none");
    
    if (status == "ok") {
        var zmawindow = window.open(serverurl, "zmapopup", "toolbar=no,location=no,directories=no,status=yes,menubar=no,scrollbars=yes,resizable=yes, width=780, height=530");
        zmawindow.focus();
    }
    else {
        alert(dgettext("plugin_zma", "An error occured while archiving")+":\n"+ ((message=="none")?dgettext("plugin_zma", 'unknown error'):message));
    }
}
