<?php
/*
 * Copyright 2005 - 2009  Zarafa B.V.
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License, version 3, 
 * as published by the Free Software Foundation with the following additional 
 * term according to sec. 7:
 *  
 * According to sec. 7 of the GNU Affero General Public License, version
 * 3, the terms of the AGPL are supplemented with the following terms:
 * 
 * "Zarafa" is a registered trademark of Zarafa B.V. The licensing of
 * the Program under the AGPL does not imply a trademark license.
 * Therefore any rights, title and interest in our trademarks remain
 * entirely with us.
 * 
 * However, if you propagate an unmodified version of the Program you are
 * allowed to use the term "Zarafa" to indicate that you distribute the
 * Program. Furthermore you may use our trademarks where it is necessary
 * to indicate the intended purpose of a product or service provided you
 * use it in accordance with honest practices in industrial or commercial
 * matters.  If you want to propagate modified versions of the Program
 * under the name "Zarafa" or "Zarafa Server", you may only do so if you
 * have a written permission by Zarafa B.V. (to acquire a permission
 * please contact Zarafa at trademark@zarafa.com).
 * 
 * The interactive user interface of the software displays an attribution
 * notice containing the term "Zarafa" and/or the logo of Zarafa.
 * Interactive user interfaces of unmodified and modified versions must
 * display Appropriate Legal Notices according to sec. 5 of the GNU
 * Affero General Public License, version 3, when you propagate
 * unmodified or modified versions of the Program. In accordance with
 * sec. 7 b) of the GNU Affero General Public License, version 3, these
 * Appropriate Legal Notices must retain the logo of Zarafa or display
 * the words "Initial Development by Zarafa" if the display of the logo
 * is not reasonably feasible for technical reasons. The use of the logo
 * of Zarafa in Legal Notices is allowed for unmodified and modified
 * versions of the software.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 *  
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

?>
<?php
/**
 * Z-Merge Archive Plugin
 *
 * Author: Sebastian Kummer <s.kummer@zarafa.com>
 *
 */

class Pluginzma extends Plugin {

function Pluginzma(){}

    function init(){
        // include generated css and js files with the configured parameters
        $this->registerHook('server.main.include.cssfiles');
        $this->registerHook('server.main.include.jsfiles');
        // trigger the entire execute event for maillistmodule
        $this->registerHook('server.module.maillistmodule.execute.before');
    }

    function execute($eventID, &$data){
        switch($eventID)
        {
            case "server.main.include.cssfiles":
                $this->includeCSS($data);
                break;
            case "server.main.include.jsfiles":
                $this->includeJS($data);
                break;
            case "server.module.maillistmodule.execute.before":
                $this->mailListModuleExecute($data['moduleObject']);
                break;

        }
    }

    // include a generated CSS version for the configuration
    function includeCSS(&$data) {
        $data['files'][] = PATH_PLUGIN_DIR."/zma/zma.config.css.php";
    }

    // include a generated JS version of the configuration
    function includeJS(&$data) {
        $data['files'][] = PATH_PLUGIN_DIR."/zma/zma.config.js.php";
    }

    // triggered on maillistmodule->execute - works with complete maillistmodule object 
    function mailListModuleExecute(&$maillist) {
        foreach($maillist->data as $action)
        {
            if(isset($action["attributes"]) && isset($action["attributes"]["type"])) {
                $store = $maillist->getActionStore($action);
                $parententryid = $maillist->getActionParentEntryID($action);
                $entryid = $maillist->getActionEntryID($action);
            
                switch($action["attributes"]["type"])
                {
                    case "zmaSendMail":
                        $result = $this->sendMail($maillist, $store, $parententryid, $entryid, $action);
                        break;
                }
            }
        }
    }


    // reads the RFC822 message from MAPI and submits it to the configured profile
    function sendMail(&$maillist, $store, $parententryid, $entryid, $action) {

        if( (!isset($action["profile"])) || 
            (!isset($GLOBALS['pluginconfig']['zma']['profiles'][$action["profile"]]))) 
            return;

        // build a connection manager from config                    
        $profilename = $action["profile"];
        $profile = $GLOBALS['pluginconfig']['zma']['profiles'][$profilename];

        // use z-merge authentication libs
        $classname = $profile['authClass'];

        // cancel if it's not possible to read the RFC822 directly from PHP-MAPI or the connector class doesn't exist
        if (!function_exists("mapi_inetmapi_imtoinet") || !class_exists($classname)) {
            $response = array("status" => "failed");

            if (!class_exists($classname))
                $response["message"]= dgettext('plugin_zma', "Z-Merge Archive: Authentication class not found for profile").": " . $profilename;
            else
                $response["message"]= dgettext('plugin_zma', "This version of Zarafa doesn't support this functionality.");

            $response["attributes"] = array("type" => "archiveResponse");
            array_push($maillist->responseData["action"], $response);
            $GLOBALS["bus"]->addData($maillist->responseData);
            return true;
        }

        // get message and addressbook         
        $message = mapi_msgstore_openentry($store, $entryid);
        $addrBook = mapi_openaddressbook($GLOBALS["mapisession"]->getSession());

        // open stream
        $stream = mapi_inetmapi_imtoinet($GLOBALS["mapisession"]->getSession(), $addrBook, $message, array());

        // try to load existing session information for this profile
        $saved_session = $this->getData($profilename);
        if (!is_array($saved_session)) $saved_session = array();

        // instantiate the authentication class
        $authClass = new $classname($profile, $saved_session);

        // build data 
        $moreData = array(
                        "user" => $GLOBALS["mapisession"]->getUserName(),
                        "archivefile" => bin2hex($parententryid) . "&" . bin2hex($entryid)
                    );

        // upload file
        $response = $authClass->uploadData($profile['archiveUri'], $stream, "archivefile.txt", "message/rfc822", $moreData);

        // save existing session information for this profile
        if ($authClass->isSessionValid())
            $this->setData($profilename, $authClass->getSessionData());

        if (isset($response['message'])) $response['message'] = dgettext('plugin_zma', $response['message']);

        $response["attributes"] = array("type" => "archiveResponse");
        array_push($maillist->responseData["action"], $response);
        $GLOBALS["bus"]->addData($maillist->responseData);
    }
}
?>